#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <unistd.h>

int main(int argc, char* argv[]){

    FILE *f1;
    FILE *f2;
    int f1_c, f1_l, f2_c, f2_l, lines, columns;

    if (argc != 3){
        printf("usage: addmx file1 file 2\n");
        exit(-1);
    }
    f1 = fopen(argv[1], "r");
    f2 = fopen(argv[2], "r");

    fscanf(f1, "%dx%d", &f1_l, &f1_c);
    fscanf(f2, "%dx%d", &f2_l, &f2_c);

    if(f1_l != f2_l || f1_c != f2_c){
        printf("Matrix sizes do not match\n");
        exit(-1);
    }
    lines = f1_l;
    columns = f1_c;

    int *f1_p = mmap(NULL, sizeof(int) * lines * columns, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, 0, 0);
    int *f2_p = mmap(NULL, sizeof(int) * lines * columns, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, 0, 0);
    int *final_m = mmap(NULL, sizeof(int) * lines * columns, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, 0, 0);

    for (int l = 0;  l<lines; l++){
        for(int c = 0;  c<columns; c++){
             fscanf(f1, "%d", &f1_p[c+columns*l]);
             fscanf(f2, "%d", &f2_p[c+columns*l]);
             final_m[c+columns*l] = 0;
        }
    }
    fclose(f1);
    fclose(f2);



    if(f1_p == MAP_FAILED || f2_p == MAP_FAILED || final_m == MAP_FAILED){
        printf("Mapping Failed\n");
        return 1;
    }
    
        //add matrix
    for (int c = 0;  c<columns; c++){
        pid_t id = fork();
        if(id == 0) {
            
            for(int l = 0;  l<lines; l++){
                final_m[c+columns*l] = f1_p[c+columns*l] + f2_p[c+columns*l];
            }
            exit(EXIT_SUCCESS);

        } if (id < 0) {
            perror("fork");
            exit(EXIT_FAILURE);
        }
    }

                //WAIT for childs
    for (int c = 0;  c<columns; c++){
        if ( waitpid(-1, NULL, 0) < 0){
            perror("Error waiting");
            exit(EXIT_FAILURE);
        }
    }

            //PRINT MATRIX
    printf("%dx%d\n", lines, columns);
    for (int l = 0;  l<lines; l++){
        for(int c = 0;  c<columns; c++){
             printf("%d ", final_m[c+columns*l]);
        }
        printf("\n");
    }
    munmap(f1_p, sizeof(f1_p));
    munmap(f2_p, sizeof(f2_p));
    munmap(final_m, sizeof(final_m));



}